==> lc-driver-3.0.4.tgz: package/test/tests.js <==
const assert = require('assert');
const {action, query, driver} = __LC_DRIVER__;
const act = action;

function $(selector) {
	return document.querySelectorAll(selector)[0];
}
describe('Set & Query Input', function() {
	const _ = (name, selector) => {
		return `#set-input-${name} ${selector}`;
	};

	it('text', function(done) {
		act.input({selector: _('text', 'input'), value: '123'}).then(() => {
			return act.input({selector: _('text', 'input[readonly]'), value: 'test'});
		}).then(() => {
			assert.equal($(_('text', 'input')).value, '123');
			assert.equal($(_('text', 'input[readonly]')).value, '');
			return query.value({selector: _('text', 'input')});
		}).then(val => {
			assert.equal(val, '123');
			return query.value({selector: _('text', 'input[readonly]')});
		}).then(val => {
			assert.equal(val, '');
			done();
		});
	});

	it('password', function(done) {
		act.input({selector: _('password', 'input'), value: '123'}).then(() => {
			assert.equal($(_('password', 'input')).value, '123');
			return query.value({selector: _('password', 'input')});
		}).then(val => {
			assert.equal(val, '123');
			done();
		});
	});

	it('email', function(done) {
		act.input({selector: _('email', 'input'), value: 'lichao@or-change.cn'}).then(() => {
			assert.equal($(_('email', 'input')).value, 'lichao@or-change.cn');
			return query.value({selector: _('email', 'input')});
		}).then(val => {
			assert.equal(val, 'lichao@or-change.cn');
			done();
		});
	});

	it('search', function(done) {
		act.input({selector: _('search', 'input'), value: 'keyword'}).then(() => {
			assert.equal($(_('search', 'input')).value, 'keyword');
			return query.value({selector: _('search', 'input')});
		}).then(val => {
			assert.equal(val, 'keyword');
			done();
		});
	});

	it('tel', function(done) {
		act.input({selector: _('tel', 'input'), value: '12345678'}).then(() => {
			assert.equal($(_('tel', 'input')).value, '12345678');
			return query.value({selector: _('tel', 'input')});
		}).then(val => {
			assert.equal(val, '12345678');
			done();
		});
	});

	it('url', function(done) {
		act.input({selector: _('url', 'input'), value: 'www.lemonce.net'}).then(() => {
			assert.equal($(_('url', 'input')).value, 'www.lemonce.net');
			return query.value({selector: _('url', 'input')});
		}).then(val => {
			assert.equal(val, 'www.lemonce.net');
			done();
		});
	});

	it('number', function(done) {
		act.input({selector: _('number', 'input#number'), value: '1'}).then(() => {
			return act.input({selector: _('number', 'input#string'), value: 'abc'});
		}).then(() => {
			assert.equal($(_('number', 'input#number')).value, '1');
			assert.equal($(_('number', 'input#string')).value, '');
			return query.value({selector: _('number', 'input#number')});
		}).then(val => {
			assert.equal(val, '1');
			return query.value({selector: _('number', 'input#string')});
		}).then(val => {
			assert.equal(val, '');
			done();
		});
	});

	it('color', function(done) {
		act.input({selector: _('color', 'input'), value: '#ffffff'}).then(() => {
			assert.equal($(_('color', 'input')).value, '#ffffff');
			return query.value({selector: _('color', 'input')});
		}).then(val => {
			assert.equal(val, '#ffffff');
			done();
		});
	});

	it('range', function(done) {
		act.input({selector: _('range', 'input'), value: '9'}).then(() => {
			assert.equal($(_('range', 'input')).value, '9');
			return query.value({selector: _('range', 'input')});
		}).then(val => {
			assert.equal(val, '9');
			done();
		});
	});

	it('date', function(done) {
		act.input({selector: _('date', 'input'), value: '2016-12-12'}).then(() => {
			assert.equal($(_('date', 'input')).value, '2016-12-12');
			return query.value({selector: _('date', 'input')});
		}).then(val => {
			assert.equal(val, '2016-12-12');
			done();
		});
	});

	it('month', function(done) {
		act.input({selector: _('month', 'input'), value: '2016-12'}).then(() => {
			assert.equal($(_('month', 'input')).value, '2016-12');
			return query.value({selector: _('month', 'input')});
		}).then(val => {
			assert.equal(val, '2016-12');
			done();
		});
	});

	it('week', function(done) {
		act.input({selector: _('week', 'input'), value: '2016-W13'}).then(() => {
			assert.equal($(_('week', 'input')).value, '2016-W13');
			return query.value({selector: _('week', 'input')});
		}).then(val => {
			assert.equal(val, '2016-W13');
			done();
		});
	});

	it('time', function(done) {
		act.input({selector: _('time', 'input'), value: '01:01'}).then(() => {
			assert.equal($(_('time', 'input')).value, '01:01');
			return query.value({selector: _('time', 'input')});
		}).then(val => {
			assert.equal(val, '01:01');
			done();
		});
	});

	it('datetime', function(done) {
		act.input({selector: _('datetime', 'input'), value: 'abcde'}).then(() => {
			assert.equal($(_('datetime', 'input')).value, 'abcde');
			return query.value({selector: _('datetime', 'input')});
		}).then(val => {
			assert.equal(val, 'abcde');
			done();
		});
	});

	it('datetime-local', function(done) {
		act.input({selector: _('datetime-local', 'input'), value: '2016-12-09T03:56'}).then(() => {
			assert.equal($(_('datetime-local', 'input')).value, '2016-12-09T03:56');
			return query.value({selector: _('datetime-local', 'input')});
		}).then(val => {
			assert.equal(val, '2016-12-09T03:56');
			done();
		});
	});

	it('checkbox', function(done) {
		act.input({selector: _('checkbox', 'input#true'), value: true})
		.then(() => {
			return act.input({selector: _('checkbox', 'input#false'), value: false});
		}).then(() => {
			assert.equal($(_('checkbox', 'input#true')).checked, true);
			assert.equal($(_('checkbox', 'input#false')).checked, false);
			return query.value({selector: _('checkbox', 'input#true')});
		}).then(val => {
			assert.equal(val, true);
			return query.value({selector: _('checkbox', 'input#false')});
		}).then(val => {
			assert.equal(val, false);
			done();
		});
	});

	it('radio', function(done) {
		act.input({selector: _('radio', 'input'), value: true}).then(() => {
			assert.equal($(_('radio', 'input')).checked, true);
			return query.value({selector: _('radio', 'input')});
		}).then(val => {
			assert.equal(val, true);
			done();
		});
	});

	it('select', function(done) {
		act.input({selector: _('select', 'select#index'), value: 1}).then(() => {
			return act.input({selector: _('select', 'select#value'), value: 'd'});
		}).then(() => {
			return act.click({selector: _('select', 'select#click option:contains(345)')});
		}).then(() => {
			assert.equal($(_('select', 'select#index')).value, 'b');
			assert.equal($(_('select', 'select#value')).value, 'd');
			assert.equal($(_('select', 'select#click')).value, 'c');
			return query.value({selector: _('select', 'select#index')});
		}).then(val => {
			assert.equal(val, 'b');
			return query.value({selector: _('select', 'select#value')});
		}).then(val => {
			assert.equal(val, 'd');
			return query.value({selector: _('select', 'select#click')});
		}).then(val => {
			assert.equal(val, 'c');
			done();
		});
	});

	it('contenteditable', function(done) {
		act.input({selector: _('contenteditable', 'div'), value: 'd'}).then(() => {
			assert.equal($(_('contenteditable', 'div')).textContent, 'abcd');
			return query.value({selector: _('contenteditable', 'div')});
		}).then(val => {
			assert.equal(val, 'abcd');
			done();
		});
	});
});

describe('Action', function() {
	const _ = selector => {
		return `${selector}`;
	};
	it('get-content', function(done) {
		act.click({selector: _('#get-content a'), value: 1}).then(() => {
			assert.equal($(_('#get-content a')).text, 'lemonce.net');
			return query.value({selector: _('#get-content a')});
		}).then(val => {
			assert.equal(val, 'lemonce.net');
			done();
		});
	});

	it('hover', function(done) {
		Promise.resolve().then(() => {
			return act.move({selector: _('#l1')});
		}).then(() => {
			return act.move({selector: _('#l2')});
		}).then(() => {
			return act.move({selector: _('#l3')});
		}).then(() => {
			assert.equal($(_('#l3')).clientHeight>0, true);
			done();
		});
	});

	it('dblclick', function(done) {
		let dblclick = false;
		$('#dblclick button').ondblclick = function() {
			dblclick = true;
		};

		act.dblclick({selector: _('#dblclick button')}).then(() => {
			assert.equal(dblclick, true);
			done();
		});
	});
});

describe('Query', function() {
	const _ = (name, selector) => {
		return `#query-${name} ${selector}`;
	};
	it('display', function(done) {
		query.display({selector: _('display', 'div')}).then(val => {
			assert.equal(val, true);	
			return query.display({selector: _('display', 'span')});
		}).then(val => {
			assert.equal(val, false);
			done();
		});
	});

	it('number', function(done) {
		query.number({selector: _('number', 'li')}).then(number => {
			assert.equal(number, 4);
			done();
		});
	});
});

describe('Keyboard', function() {
	const _ = (name, selector) => {
		return `#keyboard-${name} ${selector}`;
	};

	it('keydown', function(done) {
		let keydownCode = null;
		let keyupCode = null;
		$(_('keydown', 'input')).onkeydown = function (ev) {
			keydownCode = ev.keyCode;
		};
		$(_('keydown', 'input')).onkeyup = function (ev) {
			keyupCode = ev.keyCode;
		};
		act.click({selector: _('keydown', 'input')}).then(() => {
			return act.keydown({code: 'Enter'});
		}).then(() => {
			return act.keyup({code: 'Enter'});
		}).then(() => {
			assert.equal(keydownCode, 13);
			assert.equal(keyupCode, 13);
			done();
		});
	});

	it('keypress', function(done) {
		let keyup = false;
		let keypressCode = null;
		let keydownCode = null;
		$(_('keypress', 'input')).onkeydown = function (ev) {
			keydownCode = ev.keyCode;
		};
		$(_('keypress', 'input')).onkeyup = function () {
			keyup = true;
		};
		$(_('keypress', 'input')).onkeypress = function (ev) {
			keypressCode = ev.keyCode;
		};
		act.click({selector: _('keypress', 'input')}).then(() => {
			return act.keypress({code: 'KeyA'});
		}).then(() => {
			assert.equal(keyup, true);
			assert.equal(keypressCode, 97);
			assert.equal(keydownCode, 65);
			done();
		});
	});

	it('shift', function(done) {
		const ch = [];
		$(_('shift', 'input')).onkeypress = function (ev) {
			ch.push(ev.charCode);
		};
		act.click({selector: _('shift', 'input')}).then(() => {
			return act.keydown({code: 'ShiftLeft'});
		}).then(() => {
			return act.keypress({code: 'KeyA'});
		}).then(() => {
			return act.keyup({code: 'ShiftLeft'});
		}).then(() => {
			return act.keypress({code: 'KeyA'});
		}).then(() => {
			assert.equal(ch[0], 65);
			assert.equal(ch[1], 97);
			done();
		});
	});
});

describe('Selector', function() {
	const _ = (name, selector) => {
		return `#selector-${name} ${selector}`;
	};
	it('js', function(done) {
		window.getInput = function() {
			return $(_('js', 'input'));
		};
		act.input({selector: 'JS: window.getInput()', value: '123'}).then(() => {
			assert.equal($(_('js', 'input')).value, '123');
			done();
		});
	});

	it('xpath', function(done) {
		act.input({selector: '//section[@id="selector-xpath"]/input', value: '123'}).then(() => {
			assert.equal($(_('xpath', 'input')).value, '123');
			done();
		});
	});
	
	it('iframe', function(done) {
		act.input({selector: '#selector-iframe iframe < input', value: '123'}).then(() => {
			return query.value({selector: '#selector-iframe iframe < input'});
		}).then(val => {
			assert.equal(val, '123');
			done();
		});
	});
});

describe('Form submission', () => {
	const _ = (name, selector) => {
		return `#form-${name} ${selector}`;
	};
	it.only('button', done => {
		let submitTriggered = false;
		$(_('button', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('button', 'input'), value: '233'}).then(() => {
			return act.click({selector: _('button', 'button')});
		}).then(() => {
			assert.equal(submitTriggered, true);
			done();
		});
	});

	it('enter-with-button', done => {
		let submitTriggered = false;
		$(_('enter-with-button', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('enter-with-button', 'input'), value: '233'}).then(() => {
			return act.keypress({code: 'Enter'});
		}).then(() => {
			assert.equal(submitTriggered, true);
			done();
		});
	});

	it('enter-without-button', done => {
		let submitTriggered = false;
		$(_('enter-without-button', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('enter-without-button', 'input'), value: '233'}).then(() => {
			return act.keypress({code: 'Enter'});
		}).then(() => {
			assert.equal(submitTriggered, true);
			done();
		});
	});

	it('not-submit', done => {
		let submitTriggered = false;
		$(_('not-submit', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('not-submit', 'input'), value: '233'}).then(() => {
			return act.keypress({code: 'Enter'});
		}).then(() => {
			assert.equal(submitTriggered, false);
			done();
		});
	});
});

describe('Driver', () => {
	it('document', () => {
		assert.equal(driver.document(), window.document);

		driver.setDocumentGetter(() => {
			return document.body;
		});
		assert.equal(driver.document(), document.body);
	});

	it('access-check', done => {
		act.scroll({selector: '#driver-access-check button', accessCheck: true}).catch(err => {
			assert.equal(err.message, 'Element can not be accessed directly.');
			done();
		});
	});
});

describe('Mouse', () => {
	it('click', done => {
		act.click({selector: '#mouse-click a'}).then(() => {
			done();
		});
	});
});

describe.only('MVC framework', function() {
	it('vue', function(done) {
		setTimeout(() => {
			act.input({selector: '#mvc-vue iframe < input', value: 'item1'}).then(() => {
				return act.keypress({code: 'Enter'});
			}).then(() => {
				return query.value({selector: '#mvc-vue iframe < section ul li label'});
			}).then(val => {
				assert.equal(val, 'item1');
				return act.click({selector: '#mvc-vue iframe < section ul li button'});
			}).then(() => {
				done();
			});
		}, 500);

	});

	it('angular', function(done) {
		setTimeout(() => {
			act.input({selector: '#mvc-angular iframe < input', value: 'item1'}).then(() => {
				return act.keypress({code: 'Enter'});
			}).then(() => {
				return query.value({selector: '#mvc-angular iframe < #main ul li label'});
			}).then(val => {
				assert.equal(val, 'item1');
				return act.click({selector: '#mvc-angular iframe < #main ul li button'});
			}).then(() => {
				done();
			});
		}, 500);
	});

	it('react', function(done) {
		setTimeout(() => {
			act.input({selector: '#mvc-react iframe < input', value: 'item1'}).then(() => {
				return act.keypress({code: 'Enter'});
			}).then(() => {
				return query.value({selector: '#mvc-react iframe < #main ul li label'});
			}).then(val => {
				assert.equal(val, 'item1');
				return act.click({selector: '#mvc-react iframe < #main ul li button'});
			}).then(() => {
				done();
			});
		}, 500);
	});
});

==> lc-driver-3.0.5.tgz: package/test/tests.js <==
const assert = require('assert');
const {action, query, driver} = __LC_DRIVER__;
const act = action;

function $(selector) {
	return document.querySelectorAll(selector)[0];
}
describe('Set & Query Input', function() {
	const _ = (name, selector) => {
		return `#set-input-${name} ${selector}`;
	};

	it('text', function(done) {
		act.input({selector: _('text', 'input'), value: '123'}).then(() => {
			return act.input({selector: _('text', 'input[readonly]'), value: 'test'});
		}).then(() => {
			assert.equal($(_('text', 'input')).value, '123');
			assert.equal($(_('text', 'input[readonly]')).value, '');
			return query.value({selector: _('text', 'input')});
		}).then(val => {
			assert.equal(val, '123');
			return query.value({selector: _('text', 'input[readonly]')});
		}).then(val => {
			assert.equal(val, '');
			done();
		});
	});

	it('password', function(done) {
		act.input({selector: _('password', 'input'), value: '123'}).then(() => {
			assert.equal($(_('password', 'input')).value, '123');
			return query.value({selector: _('password', 'input')});
		}).then(val => {
			assert.equal(val, '123');
			done();
		});
	});

	it('email', function(done) {
		act.input({selector: _('email', 'input'), value: 'lichao@or-change.cn'}).then(() => {
			assert.equal($(_('email', 'input')).value, 'lichao@or-change.cn');
			return query.value({selector: _('email', 'input')});
		}).then(val => {
			assert.equal(val, 'lichao@or-change.cn');
			done();
		});
	});

	it('search', function(done) {
		act.input({selector: _('search', 'input'), value: 'keyword'}).then(() => {
			assert.equal($(_('search', 'input')).value, 'keyword');
			return query.value({selector: _('search', 'input')});
		}).then(val => {
			assert.equal(val, 'keyword');
			done();
		});
	});

	it('tel', function(done) {
		act.input({selector: _('tel', 'input'), value: '12345678'}).then(() => {
			assert.equal($(_('tel', 'input')).value, '12345678');
			return query.value({selector: _('tel', 'input')});
		}).then(val => {
			assert.equal(val, '12345678');
			done();
		});
	});

	it('url', function(done) {
		act.input({selector: _('url', 'input'), value: 'www.lemonce.net'}).then(() => {
			assert.equal($(_('url', 'input')).value, 'www.lemonce.net');
			return query.value({selector: _('url', 'input')});
		}).then(val => {
			assert.equal(val, 'www.lemonce.net');
			done();
		});
	});

	it('number', function(done) {
		act.input({selector: _('number', 'input#number'), value: '1'}).then(() => {
			return act.input({selector: _('number', 'input#string'), value: 'abc'});
		}).then(() => {
			assert.equal($(_('number', 'input#number')).value, '1');
			assert.equal($(_('number', 'input#string')).value, '');
			return query.value({selector: _('number', 'input#number')});
		}).then(val => {
			assert.equal(val, '1');
			return query.value({selector: _('number', 'input#string')});
		}).then(val => {
			assert.equal(val, '');
			done();
		});
	});

	it('color', function(done) {
		act.input({selector: _('color', 'input'), value: '#ffffff'}).then(() => {
			assert.equal($(_('color', 'input')).value, '#ffffff');
			return query.value({selector: _('color', 'input')});
		}).then(val => {
			assert.equal(val, '#ffffff');
			done();
		});
	});

	it('range', function(done) {
		act.input({selector: _('range', 'input'), value: '9'}).then(() => {
			assert.equal($(_('range', 'input')).value, '9');
			return query.value({selector: _('range', 'input')});
		}).then(val => {
			assert.equal(val, '9');
			done();
		});
	});

	it('date', function(done) {
		act.input({selector: _('date', 'input'), value: '2016-12-12'}).then(() => {
			assert.equal($(_('date', 'input')).value, '2016-12-12');
			return query.value({selector: _('date', 'input')});
		}).then(val => {
			assert.equal(val, '2016-12-12');
			done();
		});
	});

	it('month', function(done) {
		act.input({selector: _('month', 'input'), value: '2016-12'}).then(() => {
			assert.equal($(_('month', 'input')).value, '2016-12');
			return query.value({selector: _('month', 'input')});
		}).then(val => {
			assert.equal(val, '2016-12');
			done();
		});
	});

	it('week', function(done) {
		act.input({selector: _('week', 'input'), value: '2016-W13'}).then(() => {
			assert.equal($(_('week', 'input')).value, '2016-W13');
			return query.value({selector: _('week', 'input')});
		}).then(val => {
			assert.equal(val, '2016-W13');
			done();
		});
	});

	it('time', function(done) {
		act.input({selector: _('time', 'input'), value: '01:01'}).then(() => {
			assert.equal($(_('time', 'input')).value, '01:01');
			return query.value({selector: _('time', 'input')});
		}).then(val => {
			assert.equal(val, '01:01');
			done();
		});
	});

	it('datetime', function(done) {
		act.input({selector: _('datetime', 'input'), value: 'abcde'}).then(() => {
			assert.equal($(_('datetime', 'input')).value, 'abcde');
			return query.value({selector: _('datetime', 'input')});
		}).then(val => {
			assert.equal(val, 'abcde');
			done();
		});
	});

	it('datetime-local', function(done) {
		act.input({selector: _('datetime-local', 'input'), value: '2016-12-09T03:56'}).then(() => {
			assert.equal($(_('datetime-local', 'input')).value, '2016-12-09T03:56');
			return query.value({selector: _('datetime-local', 'input')});
		}).then(val => {
			assert.equal(val, '2016-12-09T03:56');
			done();
		});
	});

	it('checkbox', function(done) {
		act.input({selector: _('checkbox', 'input#true'), value: true})
		.then(() => {
			return act.input({selector: _('checkbox', 'input#false'), value: false});
		}).then(() => {
			assert.equal($(_('checkbox', 'input#true')).checked, true);
			assert.equal($(_('checkbox', 'input#false')).checked, false);
			return query.value({selector: _('checkbox', 'input#true')});
		}).then(val => {
			assert.equal(val, true);
			return query.value({selector: _('checkbox', 'input#false')});
		}).then(val => {
			assert.equal(val, false);
			done();
		});
	});

	it('radio', function(done) {
		act.input({selector: _('radio', 'input'), value: true}).then(() => {
			assert.equal($(_('radio', 'input')).checked, true);
			return query.value({selector: _('radio', 'input')});
		}).then(val => {
			assert.equal(val, true);
			done();
		});
	});

	it('select', function(done) {
		act.input({selector: _('select', 'select#index'), value: 1}).then(() => {
			return act.input({selector: _('select', 'select#value'), value: 'd'});
		}).then(() => {
			return act.click({selector: _('select', 'select#click option:contains(345)')});
		}).then(() => {
			assert.equal($(_('select', 'select#index')).value, 'b');
			assert.equal($(_('select', 'select#value')).value, 'd');
			assert.equal($(_('select', 'select#click')).value, 'c');
			return query.value({selector: _('select', 'select#index')});
		}).then(val => {
			assert.equal(val, 'b');
			return query.value({selector: _('select', 'select#value')});
		}).then(val => {
			assert.equal(val, 'd');
			return query.value({selector: _('select', 'select#click')});
		}).then(val => {
			assert.equal(val, 'c');
			done();
		});
	});

	it('contenteditable', function(done) {
		act.input({selector: _('contenteditable', 'div'), value: 'd'}).then(() => {
			assert.equal($(_('contenteditable', 'div')).textContent, 'abcd');
			return query.value({selector: _('contenteditable', 'div')});
		}).then(val => {
			assert.equal(val, 'abcd');
			done();
		});
	});
});

describe('Action', function() {
	const _ = selector => {
		return `${selector}`;
	};
	it('get-content', function(done) {
		act.click({selector: _('#get-content a'), value: 1}).then(() => {
			assert.equal($(_('#get-content a')).text, 'lemonce.net');
			return query.value({selector: _('#get-content a')});
		}).then(val => {
			assert.equal(val, 'lemonce.net');
			done();
		});
	});

	it('hover', function(done) {
		Promise.resolve().then(() => {
			return act.move({selector: _('#l1')});
		}).then(() => {
			return act.move({selector: _('#l2')});
		}).then(() => {
			return act.move({selector: _('#l3')});
		}).then(() => {
			assert.equal($(_('#l3')).clientHeight>0, true);
			done();
		});
	});

	it('dblclick', function(done) {
		let dblclick = false;
		$('#dblclick button').ondblclick = function() {
			dblclick = true;
		};

		act.dblclick({selector: _('#dblclick button')}).then(() => {
			assert.equal(dblclick, true);
			done();
		});
	});
});

describe('Query', function() {
	const _ = (name, selector) => {
		return `#query-${name} ${selector}`;
	};
	it('display', function(done) {
		query.display({selector: _('display', 'div')}).then(val => {
			assert.equal(val, true);	
			return query.display({selector: _('display', 'span')});
		}).then(val => {
			assert.equal(val, false);
			done();
		});
	});

	it('number', function(done) {
		query.number({selector: _('number', 'li')}).then(number => {
			assert.equal(number, 4);
			done();
		});
	});
});

describe('Keyboard', function() {
	const _ = (name, selector) => {
		return `#keyboard-${name} ${selector}`;
	};

	it('keydown', function(done) {
		let keydownCode = null;
		let keyupCode = null;
		$(_('keydown', 'input')).onkeydown = function (ev) {
			keydownCode = ev.keyCode;
		};
		$(_('keydown', 'input')).onkeyup = function (ev) {
			keyupCode = ev.keyCode;
		};
		act.click({selector: _('keydown', 'input')}).then(() => {
			return act.keydown({code: 'Enter'});
		}).then(() => {
			return act.keyup({code: 'Enter'});
		}).then(() => {
			assert.equal(keydownCode, 13);
			assert.equal(keyupCode, 13);
			done();
		});
	});

	it('keypress', function(done) {
		let keyup = false;
		let keypressCode = null;
		let keydownCode = null;
		$(_('keypress', 'input')).onkeydown = function (ev) {
			keydownCode = ev.keyCode;
		};
		$(_('keypress', 'input')).onkeyup = function () {
			keyup = true;
		};
		$(_('keypress', 'input')).onkeypress = function (ev) {
			keypressCode = ev.keyCode;
		};
		act.click({selector: _('keypress', 'input')}).then(() => {
			return act.keypress({code: 'KeyA'});
		}).then(() => {
			assert.equal(keyup, true);
			assert.equal(keypressCode, 97);
			assert.equal(keydownCode, 65);
			done();
		});
	});

	it('shift', function(done) {
		const ch = [];
		$(_('shift', 'input')).onkeypress = function (ev) {
			ch.push(ev.charCode);
		};
		act.click({selector: _('shift', 'input')}).then(() => {
			return act.keydown({code: 'ShiftLeft'});
		}).then(() => {
			return act.keypress({code: 'KeyA'});
		}).then(() => {
			return act.keyup({code: 'ShiftLeft'});
		}).then(() => {
			return act.keypress({code: 'KeyA'});
		}).then(() => {
			assert.equal(ch[0], 65);
			assert.equal(ch[1], 97);
			done();
		});
	});
});

describe.only('Selector', function() {
	const _ = (name, selector) => {
		return `#selector-${name} ${selector}`;
	};
	it('js', function(done) {
		window.getInput = function() {
			return $(_('js', 'input'));
		};
		act.input({selector: 'JS: window.getInput()', value: '123'}).then(() => {
			assert.equal($(_('js', 'input')).value, '123');
			done();
		});
	});

	it('xpath', function(done) {
		act.input({selector: '//section[@id="selector-xpath"]/input', value: '123'}).then(() => {
			assert.equal($(_('xpath', 'input')).value, '123');
			done();
		});
	});
	
	it('iframe', function(done) {
		act.input({selector: '#selector-iframe iframe < input', value: '123'}).then(() => {
			return query.value({selector: '#selector-iframe iframe < input'});
		}).then(val => {
			assert.equal(val, '123');
			done();
		});
	});

	it('iframe xpath', function(done) {
		act.input({selector: '/*[@id="selector-iframe"]/iframe < /input', value: '123'}).then(() => {
			return query.value({selector: '/*[@id="selector-iframe"]/iframe < /input'});
		}).then(val => {
			assert.equal(val, '123');
			done();
		});
	});
});

describe('Form submission', () => {
	const _ = (name, selector) => {
		return `#form-${name} ${selector}`;
	};
	it('button', done => {
		let submitTriggered = false;
		$(_('button', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('button', 'input'), value: '233'}).then(() => {
			return act.click({selector: _('button', 'button')});
		}).then(() => {
			assert.equal(submitTriggered, true);
			done();
		});
	});

	it('enter-with-button', done => {
		let submitTriggered = false;
		$(_('enter-with-button', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('enter-with-button', 'input'), value: '233'}).then(() => {
			return act.keypress({code: 'Enter'});
		}).then(() => {
			assert.equal(submitTriggered, true);
			done();
		});
	});

	it('enter-without-button', done => {
		let submitTriggered = false;
		$(_('enter-without-button', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('enter-without-button', 'input'), value: '233'}).then(() => {
			return act.keypress({code: 'Enter'});
		}).then(() => {
			assert.equal(submitTriggered, true);
			done();
		});
	});

	it('not-submit', done => {
		let submitTriggered = false;
		$(_('not-submit', 'form')).onsubmit = function() {
			submitTriggered = true;
			return false;
		};
		act.input({selector: _('not-submit', 'input'), value: '233'}).then(() => {
			return act.keypress({code: 'Enter'});
		}).then(() => {
			assert.equal(submitTriggered, false);
			done();
		});
	});
});

describe('Driver', () => {
	it('document', () => {
		assert.equal(driver.document(), window.document);

		driver.setDocumentGetter(() => {
			return document.body;
		});
		assert.equal(driver.document(), document.body);
	});

	it('access-check', done => {
		act.scroll({selector: '#driver-access-check button', accessCheck: true}).catch(err => {
			assert.equal(err.message, 'Element can not be accessed directly.');
			done();
		});
	});
});

describe('Mouse', () => {
	it('click', done => {
		act.click({selector: '#mouse-click a'}).then(() => {
			done();
		});
	});
});

describe('MVC framework', function() {
	it('vue', function(done) {
		setTimeout(() => {
			act.input({selector: '#mvc-vue iframe < input', value: 'item1'}).then(() => {
				return act.keypress({code: 'Enter'});
			}).then(() => {
				return query.value({selector: '#mvc-vue iframe < section ul li label'});
			}).then(val => {
				assert.equal(val, 'item1');
				return act.click({selector: '#mvc-vue iframe < section ul li button'});
			}).then(() => {
				done();
			});
		}, 500);

	});

	it('angular', function(done) {
		setTimeout(() => {
			act.input({selector: '#mvc-angular iframe < input', value: 'item1'}).then(() => {
				return act.keypress({code: 'Enter'});
			}).then(() => {
				return query.value({selector: '#mvc-angular iframe < #main ul li label'});
			}).then(val => {
				assert.equal(val, 'item1');
				return act.click({selector: '#mvc-angular iframe < #main ul li button'});
			}).then(() => {
				done();
			});
		}, 500);
	});

	it('react', function(done) {
		setTimeout(() => {
			act.input({selector: '#mvc-react iframe < input', value: 'item1'}).then(() => {
				return act.keypress({code: 'Enter'});
			}).then(() => {
				return query.value({selector: '#mvc-react iframe < #main ul li label'});
			}).then(val => {
				assert.equal(val, 'item1');
				return act.click({selector: '#mvc-react iframe < #main ul li button'});
			}).then(() => {
				done();
			});
		}, 500);
	});
});
